package digital.laboratory.platform.identifybook.controller;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import digital.laboratory.platform.common.core.constant.OSSDirectoryConstants;
import digital.laboratory.platform.common.core.util.ClassUtils;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.feign.RemoteGenerateWordService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.identifybook.dto.EntrustCaseDTO;
import digital.laboratory.platform.identifybook.emums.BusinessStatus;
import digital.laboratory.platform.identifybook.emums.PermissionGroupEnum;
import digital.laboratory.platform.identifybook.emums.TaskTypeEnum;
import digital.laboratory.platform.identifybook.entity.*;
import digital.laboratory.platform.identifybook.event.PushDataToLabsCareEvent;
import digital.laboratory.platform.identifybook.handler.AppStartupRunner;
import digital.laboratory.platform.identifybook.req.BusinessProcessRecordREQ;
import digital.laboratory.platform.identifybook.service.*;
import digital.laboratory.platform.identifybook.vo.BusinessProcessRecordVO;
import digital.laboratory.platform.sys.feign.RemoteUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationContext;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 业务流程 前端控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-05-22
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "业务流程-相关接口")
@RequestMapping("/book/businessProcessRecord")
public class BusinessProcessRecordController {

    private final RemoteUserService remoteUserService;

    private final IBusinessProcessRecordService businessProcessRecordService;
    private final IIdentifyBookRelevanceService identifyBookRelevanceService;
    private final IIdentifyBookApproveService identifyBookApproveService;
    private final IEntrustmentService entrustmentService;

    private final IEntrustmentIdentificationMaterialService entrustmentIdentificationMaterialService;
    // 根据模板身材word工具服务
    private final RemoteGenerateWordService remoteGenerateWordService;

    private final IModifySupplementApplyService modifySupplementApplyService;

    private final IReceiveIdentifyBookService receiveIdentifyBookService;

    private final ITaskNotificationService taskNotificationService;

    @ApiOperation(value = "通过条件分页查询业务流程记录列表", notes = "通过条件分页查询业务流程记录列表")
    @PostMapping("/search")
    public R<IPage<BusinessProcessRecord>> findBusinessProcessRecordPage(@RequestBody BusinessProcessRecordREQ req){
        return R.ok(businessProcessRecordService.page(req.getPage(),  null));
    }

    @ApiOperation(value = "根据业务ID查询业务流程记录", notes = "根据业务ID查询业务流程记录")
    @PostMapping("/view/{businessId}")
    public R<List<BusinessProcessRecord>> view(@PathVariable(value = "businessId") String businessId){
        List<BusinessProcessRecord> businessProcessRecordList = businessProcessRecordService.list(
                new LambdaQueryWrapper<BusinessProcessRecord>()
                        .eq(BusinessProcessRecord::getBusinessId, businessId)
                        .orderByAsc(BusinessProcessRecord::getCreateTime));
        return R.ok(businessProcessRecordList);
    }

    @ApiOperation(value = "鉴定文书审批退回修改—重新提交", notes = "鉴定文书审批退回修改——重新提交")
    @PostMapping("/identifyBookSPTHXGCXTJ")
    public R<List<BusinessProcessRecord>> identifyBookSPTHXGCXTJ(@RequestBody BusinessProcessRecordVO vo, HttpServletRequest request){
        boolean update = identifyBookRelevanceService.update(new LambdaUpdateWrapper<IdentifyBookRelevance>()
                .set(IdentifyBookRelevance::getBookStatus, vo.getBusinessStatus())
                .eq(IdentifyBookRelevance::getRelevanceBusinessId, vo.getBusinessId()));
        if(update){
            // 获取用户消息
            Principal principal = request.getUserPrincipal();
            DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

            TaskNotification taskNotification = new TaskNotification();
            BusinessProcessRecord record = new BusinessProcessRecord();
            record.setBusinessId(vo.getBusinessId());
            record.setBusinessName("鉴定文书审批退回修改-重新提交");
            record.setBusinessStatus(Integer.parseInt(vo.getBusinessStatus()));
            if (BusinessStatus.JD_WS_SH_TG.getCode().equals(vo.getBusinessStatus())){
                record.setBusinessTypeName(BusinessStatus.JD_WS_SH_TG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : "鉴定文书审批退回修改-重新提交");
                businessProcessRecordService.save(record);
                // 修改推送任务通知处理状态
                boolean flag = commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 1);
                if(flag){

                    // 设置任务通知公共信息
                    commSetTaskNotification(taskNotification, vo.getBusinessId(),
                            Integer.parseInt(TaskTypeEnum.JD_WS_SP.getCode()),
                            TaskTypeEnum.JD_WS_SP.getDesc(),
                            record.getMessage(),
                            PermissionGroupEnum.IdentifyBookSP.getCode());
                    // 推送任务通知
                    commSaveTaskNotification(taskNotification);
                }
            }
        }
        return R.ok();
    }

    @ApiOperation(value = "鉴定文书（提交/审核/审批/签发）", notes = "鉴定文书（提交/审核/审批/签发）")
    @Transactional
    @PostMapping("/identifyBookTJSHSPQF")
    public R<Object> identifyBookTJSHSPQF(@RequestBody BusinessProcessRecordVO vo, HttpServletRequest request){
        boolean update = identifyBookRelevanceService.update(new LambdaUpdateWrapper<IdentifyBookRelevance>()
                .set(IdentifyBookRelevance::getBookStatus, vo.getBusinessStatus())
                .eq(IdentifyBookRelevance::getRelevanceBusinessId, vo.getBusinessId()));

        if(update){
            // 获取用户消息
            Principal principal = request.getUserPrincipal();
            DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

            TaskNotification taskNotification = new TaskNotification();
            BusinessProcessRecord record = new BusinessProcessRecord();
            record.setBusinessId(vo.getBusinessId());
            record.setBusinessName("鉴定文书提交审核审批签发");
            record.setBusinessStatus(Integer.parseInt(vo.getBusinessStatus()));

            if(BusinessStatus.JD_WS_TJ_SH.getCode().equals(vo.getBusinessStatus())){

                // 提交审核
                record.setBusinessTypeName(BusinessStatus.JD_WS_TJ_SH.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_TJ_SH.getDesc());

                // 主要处理是否是退回修改的业务
                boolean flag = commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 1);
                // 设置任务通知公共信息
                commSetTaskNotification(taskNotification, vo.getBusinessId(),
                        Integer.parseInt(TaskTypeEnum.JD_WS_SH.getCode()),
                        TaskTypeEnum.JD_WS_SH.getDesc(),
                        record.getMessage(),
                        PermissionGroupEnum.IdentifyBookSH.getCode());
                if(!flag){

                    taskNotification.setSubmitterUserId(dlpUser.getId());
                    taskNotification.setSubmitterUserName(dlpUser.getName());
                    Entrustment entrustment = entrustmentService.getById(vo.getBusinessId());
                    taskNotification.setAcceptNo(entrustment.getAcceptNo());
                    //CaseEvent caseEvent = caseEventService.getById(entrustment.getCaseId());
                    taskNotification.setCaseName(entrustment.getCaseName());
                }


                // 推送任务通知
                commSaveTaskNotification(taskNotification);

            } else if (BusinessStatus.JD_WS_SH_TG.getCode().equals(vo.getBusinessStatus())){
                // 审核通过-保存或更新鉴定文书审审批表
                IdentifyBookApprove one = new IdentifyBookApprove();
                one.setRelevanceBusinessId(vo.getBusinessId());
                one.setProcessUserBy(dlpUser.getId());
                one.setProcessUserName(dlpUser.getName());
                one.setProcessTime(new Date());
                one.setProcessMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : "审核通过");
                one.setAnnexType(vo.getApproveVO().getAnnexType());
                one.setBookType(vo.getApproveVO().getBookType());
                one.setInspectionReportNumber(vo.getApproveVO().getInspectionReportNumber());
                one.setOtherNumber(vo.getApproveVO().getOtherNumber());
                one.setIsQualified(vo.getApproveVO().getIsQualified());

                identifyBookApproveService.saveOrUpdate(one);

                record.setBusinessTypeName(BusinessStatus.JD_WS_SH_TG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_SH_TG.getDesc());

                // 修改推送任务通知处理状态
                boolean flag = commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 1);
                if(flag){

                    // 设置任务通知公共信息
                    commSetTaskNotification(taskNotification, vo.getBusinessId(),
                            Integer.parseInt(TaskTypeEnum.JD_WS_SP.getCode()),
                            TaskTypeEnum.JD_WS_SP.getDesc(),
                            record.getMessage(),
                            PermissionGroupEnum.IdentifyBookSP.getCode());
                    // 推送任务通知
                    commSaveTaskNotification(taskNotification);
                }

            } else if (BusinessStatus.JD_WS_SH_WTG.getCode().equals(vo.getBusinessStatus())){
                // 审核未通过
                record.setBusinessTypeName(BusinessStatus.JD_WS_SH_WTG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_SH_WTG.getDesc());

                // 修改推送任务通知处理状态
                if(commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), -1)){

                }

            } else if (BusinessStatus.JD_WS_SH_TH_XG.getCode().equals(vo.getBusinessStatus())){
                // 审核-退回修改
                record.setBusinessTypeName(BusinessStatus.JD_WS_SH_TH_XG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_SH_TH_XG.getDesc());
                // 修改推送任务通知处理状态
                boolean flag = commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 2);
                if(flag){
                    // 设置任务通知公共信息
                    commSetTaskNotification(taskNotification, vo.getBusinessId(),
                            Integer.parseInt(TaskTypeEnum.JD_WS_SH_TH_XG.getCode()),
                            TaskTypeEnum.JD_WS_SH_TH_XG.getDesc(),
                            record.getMessage(),
                            PermissionGroupEnum.IdentifyBookProductionRrSubmissionProcess.getCode());
                    // 推送任务通知
                    commSaveTaskNotification(taskNotification);
                }

            } else if (BusinessStatus.JD_WS_SP_QF_TG.getCode().equals(vo.getBusinessStatus())){
                // TODO 审批签发成功--更新鉴定文书并生成鉴定文书审批表
                IdentifyBookApprove one = identifyBookApproveService.list(new LambdaQueryWrapper<IdentifyBookApprove>()
                        .eq(IdentifyBookApprove::getRelevanceBusinessId, vo.getBusinessId())
                        .orderByDesc(IdentifyBookApprove::getCreateTime)).get(0);

                one.setRelevanceBusinessId(vo.getBusinessId());
                one.setApproveUserBy(dlpUser.getId());
                one.setApproveUserName(dlpUser.getName());
                one.setApproveTime(new Date());
                one.setApproveMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : "审批通过");
                this.generateIdentifyBookApprove(one);

                record.setBusinessTypeName(BusinessStatus.JD_WS_SP_QF_TG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_SP_QF_TG.getDesc());
                // 审批签发通过-通知鉴定系统
                //remoteIdentificationService.putSetReportApplovalPassed(vo.getBusinessId()); 注释掉以前的通知方式

                // 修改推送任务通知处理状态
                boolean flag = commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 1);
                if(flag){

                    // 设置任务通知公共信息
                    commSetTaskNotification(taskNotification, vo.getBusinessId(),
                            Integer.parseInt(TaskTypeEnum.JD_WS_QF_LQ.getCode()),
                            TaskTypeEnum.JD_WS_QF_LQ.getDesc(),
                            record.getMessage(),
                            PermissionGroupEnum.IdentifyBookQF.getCode());
                    // 推送任务通知
                    commSaveTaskNotification(taskNotification);

                    // 发布推送事件
//                    applicationContext.publishEvent(new PushDataToLabsCareEvent(this, vo.getBusinessId(), Collections.synchronizedSet(CollUtil.newHashSet( 6))));
//                    applicationContext.publishEvent(new PushDataToLabsCareEvent(this, vo.getBusinessId(), 6));
//                    applicationContext.publishEvent(new PushDataToLabsCareEvent(this, vo.getBusinessId(), 7));
                }

            } else if (BusinessStatus.JD_WS_SP_QF_WTG.getCode().equals(vo.getBusinessStatus())){
                // 审批签发未通过
                record.setBusinessTypeName(BusinessStatus.JD_WS_SP_QF_WTG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_SP_QF_WTG.getDesc());

                // 修改推送任务通知处理状态
                if(commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), -1)){

                }

            } else if (BusinessStatus.JD_WS_SP_QF_TH_XG.getCode().equals(vo.getBusinessStatus())){
                // 审批签发退回修改
                record.setBusinessTypeName(BusinessStatus.JD_WS_SP_QF_TH_XG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_SP_QF_TH_XG.getDesc());

                // 修改推送任务通知处理状态
                boolean flag = commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 2);
                if(flag){
                    // 设置任务通知公共信息
                    commSetTaskNotification(taskNotification, vo.getBusinessId(),
                            Integer.parseInt(TaskTypeEnum.JD_WS_SP_TH_XG.getCode()),
                            TaskTypeEnum.JD_WS_SP_TH_XG.getDesc(),
                            record.getMessage(),
                            PermissionGroupEnum.IdentifyBookProductionRrSubmissionProcess.getCode());
                    // 推送任务通知
                    commSaveTaskNotification(taskNotification);
                }

            }else if (BusinessStatus.JD_WS_YLQ.getCode().equals(vo.getBusinessStatus())){
                // TODO 领取成功--生成鉴定文书领取信息
                ReceiveIdentifyBook receiveIdentifyBook = new ReceiveIdentifyBook();
                BeanUtil.copyProperties(vo, receiveIdentifyBook);
                if (receiveIdentifyBookService.count(Wrappers.<ReceiveIdentifyBook>lambdaQuery().eq(ReceiveIdentifyBook::getRelevanceBusinessId, vo.getBusinessId())) > 0) {
                    return R.failed("文书已被领取，请勿重复领取！");
                }
                receiveIdentifyBook.setRelevanceBusinessId(vo.getBusinessId());
                receiveIdentifyBook.setOperatorUserId(dlpUser.getId());
                receiveIdentifyBook.setOperatorUserName(dlpUser.getName());
                receiveIdentifyBookService.save(receiveIdentifyBook);

                record.setBusinessTypeName(BusinessStatus.JD_WS_YLQ.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_YLQ.getDesc());

                // 修改推送任务通知处理状态
                if(commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 1)){

                }

            } else {
                return R.failed("参数-业务状态未定义，请确认传入的业务状态是否正确或联系管理员！！！");
            }
            if(businessProcessRecordService.save(record)){
                return R.ok();
            }
        }
        return R.failed("系统异常，请联系管理员！！！");
    }

    @ApiOperation(value = "鉴定文书修改/补充（提交/审核/审批）", notes = "鉴定文书修改/补充（提交/审核/审批）")
    @Transactional
    @PostMapping("/identifyBookModifySupplementTJSHSP")
    public R<String> identifyBookModifySupplementTJSHSP(@RequestBody BusinessProcessRecordVO vo, HttpServletRequest request){
        boolean update = modifySupplementApplyService.update(new LambdaUpdateWrapper<ModifySupplementApply>()
                .set(ModifySupplementApply::getStatus, vo.getBusinessStatus())
                .eq(ModifySupplementApply::getId, vo.getId()));
        if(update){
            // 获取用户消息
            Principal principal = request.getUserPrincipal();
            DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

            TaskNotification taskNotification = new TaskNotification();

            ModifySupplementApply modifySupplementApply = modifySupplementApplyService.getById(vo.getId());
            BusinessProcessRecord record = new BusinessProcessRecord();
            record.setBusinessId(vo.getBusinessId());
            record.setBusinessName("鉴定文书修改/补充申请");
            record.setBusinessStatus(Integer.parseInt(vo.getBusinessStatus()));
            if(BusinessStatus.JD_WS_XG_BC_SQ.getCode().equals(vo.getBusinessStatus())) {
                // 提交审核
                record.setBusinessTypeName(BusinessStatus.JD_WS_XG_BC_SQ.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_XG_BC_SQ.getDesc());

                // 设置任务通知公共信息
                commSetTaskNotification(taskNotification, vo.getBusinessId(),
                        Integer.parseInt(TaskTypeEnum.JD_WS_XG_BC_SH.getCode()),
                        TaskTypeEnum.JD_WS_XG_BC_SH.getDesc(),
                        record.getMessage(),
                        PermissionGroupEnum.IdentifyBookModificationAndSupplementationSH.getCode());
                taskNotification.setSubmitterUserId(dlpUser.getId());
                taskNotification.setSubmitterUserName(dlpUser.getName());
                Entrustment entrustment = entrustmentService.getById(modifySupplementApply.getEntrustmentId());
                taskNotification.setAcceptNo(entrustment.getAcceptNo());
                //CaseEvent caseEvent = caseEventService.getById(entrustment.getCaseId());
                //taskNotification.setCaseName(caseEvent.getCaseName());
                taskNotification.setCaseName(entrustment.getCaseName());
                // 推送任务通知
                commSaveTaskNotification(taskNotification);
            } else if (BusinessStatus.JD_WS_XG_BC_SH_TG.getCode().equals(vo.getBusinessStatus())){
                // 审核通过 设置并修改授权审核信息
                modifySupplementApply.setEmpowerBySignDate(new Date());
                modifySupplementApply.setEmpowerBySignOpinion(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : "审核通过");
                modifySupplementApplyService.updateById(modifySupplementApply);

                record.setBusinessTypeName(BusinessStatus.JD_WS_XG_BC_SH_TG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_XG_BC_SH_TG.getDesc());

                // 修改推送任务通知处理状态
                boolean flag = commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 1);
                if(flag){

                    // 设置任务通知公共信息
                    commSetTaskNotification(taskNotification, vo.getBusinessId(),
                            Integer.parseInt(TaskTypeEnum.JD_WS_XG_BC_SP.getCode()),
                            TaskTypeEnum.JD_WS_XG_BC_SP.getDesc(),
                            record.getMessage(),
                            PermissionGroupEnum.IdentifyBookModificationAndSupplementationSP.getCode());
                    Entrustment entrustment = entrustmentService.getById(modifySupplementApply.getEntrustmentId());
                    taskNotification.setCaseName(entrustment.getCaseName());
                    // 推送任务通知
                    commSaveTaskNotification(taskNotification);
                }
            } else if (BusinessStatus.JD_WS_XG_BC_SH_WTG.getCode().equals(vo.getBusinessStatus())){
                // 审核未通过
                record.setBusinessTypeName(BusinessStatus.JD_WS_XG_BC_SH_WTG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_XG_BC_SH_WTG.getDesc());
                // 修改推送任务通知处理状态
                if(commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), -1)){

                }
            } else if (BusinessStatus.JD_WS_XG_BC_SP_TG.getCode().equals(vo.getBusinessStatus())){
                // 审批通过 设置并修改技术人员审批信息
                modifySupplementApply.setTechnologyBySignDate(new Date());
                modifySupplementApply.setTechnologyBySignOpinion(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : "审核通过");
                // 生成鉴定文书修改补充申请表
                this.generateModifySupplementApply(modifySupplementApply);
                record.setBusinessTypeName(BusinessStatus.JD_WS_XG_BC_SP_TG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_XG_BC_SP_TG.getDesc());
                // 修改推送任务通知处理状态
                if(commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), 1)){
                }
                if(modifySupplementApply.getUpdateWay() == 2){
                    // 修改鉴定文书业务状态未 收回原件-重新制作
                    identifyBookRelevanceService.update(new LambdaUpdateWrapper<IdentifyBookRelevance>()
                            .set(IdentifyBookRelevance::getBookStatus, BusinessStatus.JD_WS_XG_BC_SH_YJ_CX_ZZ.getCode())
                            .eq(IdentifyBookRelevance::getRelevanceBusinessId, modifySupplementApply.getEntrustmentId()));
                }


            } else if (BusinessStatus.JD_WS_XG_BC_SP_WTG.getCode().equals(vo.getBusinessStatus())){
                // 审批未通过
                record.setBusinessTypeName(BusinessStatus.JD_WS_XG_BC_SP_WTG.getDesc());
                record.setMessage(StringUtils.isNotEmpty(vo.getMessage()) ? vo.getMessage() : BusinessStatus.JD_WS_XG_BC_SP_WTG.getDesc());
                // 修改推送任务通知处理状态
                if(commUpdateTaskNotification(taskNotification, dlpUser, vo.getBusinessId(), -1)){

                }
            } else {
                return R.failed("参数-业务状态未定义，请确认传入的业务状态是否正确或联系管理员！！！");
            }
            if(businessProcessRecordService.save(record)){
                return R.ok();
            }
        }
        return R.failed("系统异常，请联系管理员！！！");
    }

    //获取鉴定书的处理记录，按ID排序
    @ApiOperation(value ="获取鉴定书的处理记录，按ID desc排序",notes = "获取鉴定书的处理记录，按ID desc排序")
    @GetMapping("/getIdentifyBookProcessRecord")
    public R getIdentifyBookProcessRecord(String businessId){
        List<BusinessProcessRecord> ret= businessProcessRecordService.list(Wrappers.<BusinessProcessRecord>lambdaQuery()
                .eq(BusinessProcessRecord::getBusinessId,businessId)
                .orderByDesc(BusinessProcessRecord::getId));
        return R.ok(ret);
    }
    /**
     *  TODO 生成鉴定文书审批表并保存
     * @param one 鉴定文书审批表对象
     */
    private void generateIdentifyBookApprove(IdentifyBookApprove one) {
        try {
            EntrustCaseDTO entrustCaseDTO = entrustmentService.getEntrustCaseDTOById(one.getRelevanceBusinessId());
            IdentifyBookRelevance identifyBookRelevance = identifyBookRelevanceService.getOne(new LambdaQueryWrapper<IdentifyBookRelevance>()
                    .eq(IdentifyBookRelevance::getRelevanceBusinessId, one.getRelevanceBusinessId()));

            // 模板
//            String templateFileName = "NADL-Shaanxi-CX-22-01-2021鉴定文书审批表001.doc";
            String templateFileName = AppStartupRunner.getCfg("identifyBookApproveTemplate");
            // 文件类型
            String fileType = ".docx"; // 文件类型
            // 文件版本号
            String version = "v1.0";
            // 文件名
            String fileName = "鉴定文书审批表-" + version;

            // 文件路径 + 业务ID +文件名
            String pathFileName = OSSDirectoryConstants.IDENTIFY_BOOK_DIRECTORY + "/" + one.getRelevanceBusinessId() + "/bookApprove/" + fileName;

            Map<String, Object> parameter = ClassUtils.objectToMap(entrustCaseDTO);
            parameter.put("bookYear", identifyBookRelevance.getYear());
            parameter.put("bookSerialNumber", identifyBookRelevance.getSerialNumber());
            one.setYear(identifyBookRelevance.getYear());
            one.setSerialNumber(identifyBookRelevance.getSerialNumber());

            // 获取样本数量
            long sampleNum = entrustmentIdentificationMaterialService.count(new LambdaQueryWrapper<EntrustmentIdentificationMaterial>()
                    .eq(EntrustmentIdentificationMaterial::getEntrustmentId, one.getRelevanceBusinessId()));
            parameter.put("sampleNum", sampleNum);

            one.setPath(pathFileName + fileType);
            one.setFileName(fileName + fileType);
            if(StringUtils.isEmpty(one.getYear())){
                LocalDateTime localDateTime = LocalDateTime.ofInstant(new Date().toInstant(), ZoneId.systemDefault());
                String year = LocalDateTimeUtil.format(localDateTime, "yyyy");
                List<IdentifyBookApprove> list = identifyBookApproveService.list(new LambdaQueryWrapper<IdentifyBookApprove>()
                        .eq(IdentifyBookApprove::getYear, year)
                        .orderByDesc(IdentifyBookApprove::getSerialNumber));
                if(CollectionUtils.isNotEmpty(list)){
                    int num = Integer.parseInt(list.get(0).getSerialNumber());
                    one.setYear(year);
                    one.setSerialNumber(StrUtil.fillBefore((num + 1) + "", '0', 5));
                } else {
                    one.setYear(year);
                    one.setSerialNumber(StrUtil.fillBefore( "1", '0', 5));
                }

            }
            parameter.put("year", one.getYear());
            parameter.put("serialNumber", one.getSerialNumber());
//            parameter.put("identificationRequestsDesc", IdentifyRequireEnum.findEnumByCode(entrustCaseDTO.getIdentificationRequests()).getDesc());
            parameter.put("otherIdentificationRequests", StringUtils.isNotEmpty(entrustCaseDTO.getOtherIdentificationRequests())? "其它：" + entrustCaseDTO.getOtherIdentificationRequests(): "");

            // 文书类型：（1：检验报告，2：其它）
            parameter.put("inspectionReportFlag", false);
            parameter.put("otherFlag", false);
            if(StringUtils.isNotEmpty(one.getBookType())){
                String[] bookTypeArr = one.getBookType().split(",");
                for (String s : bookTypeArr) {
                    if (Integer.parseInt(s) == 1) {
                        parameter.put("inspectionReportFlag", true);
                        parameter.put("inspectionReportNumber", one.getInspectionReportNumber());
                    } else if (Integer.parseInt(s) == 2) {
                        parameter.put("otherFlag", true);
                        parameter.put("otherNumber", one.getOtherNumber());
                    }
                }
            }

            // 附件（1：检验记录（含原始记录，检验图表）、2：鉴定事项确认书、3：鉴定委托书或鉴定聘请书、4：检材照片、5：送检人证件复印件、6：原鉴定文书）
            parameter.put("jyJlFlag", false);
            parameter.put("jdSxQrsFlag", false);
            parameter.put("jdWtsFlag", false);
            parameter.put("jcZpFlag", false);
            parameter.put("sjrZjFyjFlag", false);
            parameter.put("yJdWsFlag", false);
            if(StringUtils.isNotEmpty(one.getAnnexType())){
                String[] annexTypeArr = one.getAnnexType().split(",");
                for (String s : annexTypeArr) {
                    int num = Integer.parseInt(s);
                    if (num == 1) {
                        parameter.put("jyJlFlag", true);
                    } else if (num == 2) {
                        parameter.put("jdSxQrsFlag", true);
                    } else if (num == 3) {
                        parameter.put("jdWtsFlag", true);
                    } else if (num == 4) {
                        parameter.put("jcZpFlag", true);
                    } else if (num == 5) {
                        parameter.put("sjrZjFyjFlag", true);
                    } else if (num == 6) {
                        parameter.put("yJdWsFlag", true);
                    }
                }
            }

            // 是否合格（1：检验方法、2：检验过程、3：检验图谱、4：检验结果）
            parameter.put("jyFfFlag", false);
            parameter.put("jyGcFlag", false);
            parameter.put("jyTpFlag", false);
            parameter.put("jyJgFlag", false);
            if(StringUtils.isNotEmpty(one.getIsQualified())){
                String[] isQualifiedArr = one.getIsQualified().split(",");
                for (String s : isQualifiedArr) {
                    int num = Integer.parseInt(s);
                    if (num == 1) {
                        parameter.put("jyFfFlag", true);
                    } else if (num == 2) {
                        parameter.put("jyGcFlag", true);
                    } else if (num == 3) {
                        parameter.put("jyTpFlag", true);
                    } else if (num == 4) {
                        parameter.put("jyJgFlag", true);
                    }
                }
            }
            // 鉴定文书审核意见
            parameter.put("processMessage", one.getProcessMessage());
            // 鉴定文书审核日期
            parameter.put("processTime", LocalDateTimeUtil.format(
                    LocalDateTime.ofInstant(one.getProcessTime().toInstant(), ZoneId.systemDefault()), "yyyy年MM月dd日"));
            // 鉴定文书审批意见
            parameter.put("approveMessage", one.getApproveMessage());
            // 鉴定文书审批日期
            parameter.put("approveTime", LocalDateTimeUtil.format(
                    LocalDateTime.ofInstant(one.getApproveTime().toInstant(), ZoneId.systemDefault()), "yyyy年MM月dd日"));
            // 备注
            parameter.put("remarks", one.getRemarks());

            // 生成鉴定文书审批表
            R r = remoteGenerateWordService.generateWord(templateFileName, "", pathFileName, fileType,
                    "", null, false, parameter);
            if(r != null && r.getCode() == 0){
                identifyBookApproveService.updateById(one);
            } else {
                throw new IllegalArgumentException(r.getMsg());
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * TODO 生成鉴定文书修改/补充申请表
     * @param modifySupplementApply 修改补充对象
     */
    private void generateModifySupplementApply(ModifySupplementApply modifySupplementApply){

        try {
            IdentifyBookRelevance identifyBookRelevance = identifyBookRelevanceService.getOne(new LambdaQueryWrapper<IdentifyBookRelevance>()
                    .eq(IdentifyBookRelevance::getRelevanceBusinessId, modifySupplementApply.getEntrustmentId()));
            Entrustment entrustment = entrustmentService.getById(modifySupplementApply.getEntrustmentId());

            Map<String, Object> parameter = ClassUtils.objectToMap(modifySupplementApply);
            parameter.put("bookYear", identifyBookRelevance.getYear());
            parameter.put("bookSerialNumber", identifyBookRelevance.getSerialNumber());
            parameter.put("clientOrgName", entrustment.getClientOrgName());
            if(StringUtils.isNotEmpty(modifySupplementApply.getEntrustBy()) && StringUtils.isEmpty(modifySupplementApply.getEntrustUserName())){
                parameter.put("entrustUserName", remoteUserService.innerGetById(modifySupplementApply.getEntrustBy()).getData().getName());
            } else {
                parameter.put("entrustUserName", modifySupplementApply.getEntrustUserName());
            }

            if(ObjectUtils.isNotEmpty(modifySupplementApply.getApplyTime())){
                parameter.put("applyTime", LocalDateTimeUtil.format(
                        LocalDateTime.ofInstant(modifySupplementApply.getApplyTime().toInstant(), ZoneId.systemDefault()), "yyyy年MM月dd日"));
            }
//            parameter.put("updateUserName", remoteUserService.innerGetById(modifySupplementApply.getUpdateBy()).getData().getName());
            parameter.put("updateUserName", modifySupplementApply.getUpdateBy());
            if(ObjectUtils.isNotEmpty(modifySupplementApply.getUpdateTime())){
                parameter.put("updateTime", LocalDateTimeUtil.format(
                        LocalDateTime.ofInstant(modifySupplementApply.getUpdateTime().toInstant(), ZoneId.systemDefault()), "yyyy年MM月dd日"));
            }


            parameter.put("xgCdWjFlag", false);
            parameter.put("shYjCxCjJdWsFlag", false);
            if(modifySupplementApply.getUpdateWay() == 1){
                parameter.put("xgCdWjFlag", true);
            }else{
                parameter.put("shYjCxCjJdWsFlag", true);
            }
            if(ObjectUtils.isNotEmpty(modifySupplementApply.getEmpowerBySignDate())){
                parameter.put("empowerBySignDate", LocalDateTimeUtil.format(
                        LocalDateTime.ofInstant(modifySupplementApply.getEmpowerBySignDate().toInstant(), ZoneId.systemDefault()), "yyyy年MM月dd日"));
            }
            if(ObjectUtils.isNotEmpty(modifySupplementApply.getTechnologyBySignDate())){
                parameter.put("technologyBySignDate", LocalDateTimeUtil.format(
                        LocalDateTime.ofInstant(modifySupplementApply.getTechnologyBySignDate().toInstant(), ZoneId.systemDefault()), "yyyy年MM月dd日"));
            }


            // 获取模板文件名
            String templateFileName = AppStartupRunner.getCfg("identifyBookModifySupplementApplyTemplate");
            // 文件类型
            String fileType = ".docx"; // 文件类型
            // 文件版本号
            String version = "v1.0";
            // 文件名
            String fileName = "鉴定文书修改补充申请-" + version;

            // 文件路径 + 业务ID +文件名
            String pathFileName = OSSDirectoryConstants.IDENTIFY_BOOK_DIRECTORY + "/" + modifySupplementApply.getEntrustmentId() + "/bookModifyApply/" + fileName;
            // 生成鉴定文书审批表
            R r = remoteGenerateWordService.generateWord(templateFileName, "", pathFileName, fileType,
                    "", null, false, parameter);
            if(r != null && r.getCode() == 0){
                modifySupplementApply.setPath(pathFileName + fileType);
                modifySupplementApply.setFileName(fileName + fileType);
                modifySupplementApplyService.updateById(modifySupplementApply);
            } else {
                throw new IllegalArgumentException(r.getMsg());
            }

        }catch (Exception e){
            e.printStackTrace();
        }
    }
    /**
     * 设置任务通知公共信息
     * @param taskNotification 任务通知信息
     * @param businessId 业务ID
     * @param type 任务类型
     * @param title 任务标题
     * @param content 任务类容
     * @param permissionGroup 任务代办人员组
     */
    private void commSetTaskNotification(TaskNotification taskNotification, String businessId,
                                                int type, String title, String content, String permissionGroup){
        taskNotification.setBusinessId(businessId);
        taskNotification.setType(type);
        taskNotification.setTitle(title);
        taskNotification.setContent(content);
        taskNotification.setPermissionGroup(permissionGroup);
    }

    /**
     * 修改推送任务通知处理状态
     * @param taskNotification 任务通知基本信息
     * @return  R<Object>
     */
    private boolean commUpdateTaskNotification(TaskNotification taskNotification, DLPUser dlpUser, String businessId, int status){

        // 查询并 更新任务为已处理
        TaskNotification notification = taskNotificationService.getOne(new LambdaQueryWrapper<TaskNotification>()
                .eq(TaskNotification::getBusinessId, businessId)
                .eq(TaskNotification::getStatus, 0)
                .in(TaskNotification::getPermissionGroup, dlpUser.getPermissions()));
        if(ObjectUtils.isNotEmpty(notification)){
            notification.setStatus(status);
            notification.setTransactionPersonnelId(dlpUser.getId());
            notification.setTransactionPersonnelName(dlpUser.getName());

            taskNotification.setSubmitterUserId(notification.getSubmitterUserId());
            taskNotification.setSubmitterUserName(notification.getSubmitterUserName());
            taskNotification.setAcceptNo(notification.getAcceptNo());
            taskNotification.setCaseName(notification.getCaseName());
            return taskNotificationService.updateById(notification);
        }
        return false;

    }
    /**
     * 推送任务通知
     * @param taskNotification
     * @return
     */
    private R<Object> commSaveTaskNotification(TaskNotification taskNotification){

        // 设置任务通知
        boolean save = taskNotificationService.save(taskNotification);
        if(!save){
            return R.failed("推送任务通知失败");
        }
        return R.ok();
    }
}
