package digital.laboratory.platform.identifybook.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.identifybook.dto.EntrustmentDTO;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.req.EntrustmentREQ;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 委托 前端控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-05-17
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "委托数据-相关接口")
@RequestMapping("/book/entrustment")
public class EntrustmentController {

    private final IEntrustmentService entrustmentService;

    @ApiOperation(value = "根据委托id查询委托相关详情", notes = "根据委托id查询委托相关详情")
    @GetMapping("/view/{entrustId}")
    public R<EntrustmentDTO> view(@PathVariable(value = "entrustId") String entrustId){
        EntrustmentDTO entrustmentDTO = entrustmentService.getEntrustmentDTO(entrustId);
        return R.ok(entrustmentDTO);
    }

    @ApiOperation(value = "根据条件分页查询委托数据列表", notes = "根据条件分页查询委托数据列表")
    @PostMapping("/findEntrustPage")
    public R<IPage<EntrustmentDTO>> findEntrustPage(@RequestBody EntrustmentREQ req){
        List<Entrustment> list = entrustmentService.list();
        List<String> entrustIdList=new ArrayList<>();
        if(list.size()>0){
            entrustIdList= list.stream().map(s -> s.getId()).collect(Collectors.toList());
        }
        entrustmentService.synchronizationTestSysData(entrustIdList);//同步远程数据，同步后的数据不会再同步
        return R.ok(entrustmentService.findEntrustPage(req.getPage(), req));
    }


}
