package digital.laboratory.platform.identifybook.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.entity.IdentifyBookApprove;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import digital.laboratory.platform.identifybook.service.IIdentifyBookApproveService;
import digital.laboratory.platform.identifybook.utils.Word2PdfAsposeUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.activation.MimetypesFileTypeMap;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 鉴定文书审批表  前端控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-05-25
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "鉴定文书审批表相关接口")
@RequestMapping("/book/identifyBookApprove")
public class IdentifyBookApproveController {

    private final OssFile ossFile;
    private final IIdentifyBookApproveService identifyBookApproveService;
    private final IEntrustmentService entrustmentService;

    @ApiOperation(value = "根据委托ID获取鉴定文书审核审批详情", notes = "根据委托ID获取鉴定文书审核审批详情")
    @GetMapping("/view/{entrustId}")
    public R<IdentifyBookApprove> view(@PathVariable(value = "entrustId") String entrustId, HttpServletResponse response){
        try {
            Entrustment entrustment = entrustmentService.getById(entrustId);
            List<String> entrustIds = new ArrayList<>();
            List<Entrustment> entrustmentList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                    .select(Entrustment::getId)
                    //.eq(Entrustment::getIdentificationId, entrustment.getIdentificationId())
                    .eq(Entrustment::getId, entrustment.getId())
            );
            if(CollectionUtils.isNotEmpty(entrustmentList)){
                entrustIds.addAll(entrustmentList.stream().map(m -> m.getId()).collect(Collectors.toList()));
            }
            List<IdentifyBookApprove> identifyBookApproveList = identifyBookApproveService.list(new LambdaQueryWrapper<IdentifyBookApprove>()
                    .in(IdentifyBookApprove::getRelevanceBusinessId, entrustIds)
                    .orderByDesc(IdentifyBookApprove::getCreateTime));

            return R.ok(CollectionUtils.isNotEmpty(identifyBookApproveList)? identifyBookApproveList.get(0): null);
        }catch (Exception e){
            System.out.println("获取鉴定文书审批表异常" + e.getMessage());
            e.printStackTrace();
        }
        return R.failed("系统异常！！！");
    }

    @ApiOperation(value = "根据委托ID获取鉴定文书审批表文件", notes = "根据委托ID获取鉴定文书审批表文件")
    @GetMapping("/getIdentifyBookApprove/{entrustId}")
    public void getIdentifyBookApprove(@PathVariable(value = "entrustId") String entrustId, HttpServletResponse response){
        try {
            Entrustment entrustment = entrustmentService.getById(entrustId);
            List<String> entrustIds = new ArrayList<>();
            List<Entrustment> entrustmentList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                    .select(Entrustment::getId)
                    //.eq(Entrustment::getIdentificationId, entrustment.getIdentificationId())
                    .eq(Entrustment::getId, entrustment.getId())
            );
            if(CollectionUtils.isNotEmpty(entrustmentList)){
                entrustIds.addAll(entrustmentList.stream().map(m -> m.getId()).collect(Collectors.toList()));
            }
            List<IdentifyBookApprove> identifyBookApproveList = identifyBookApproveService.list(new LambdaQueryWrapper<IdentifyBookApprove>()
                    .in(IdentifyBookApprove::getRelevanceBusinessId, entrustIds)
                    .orderByDesc(IdentifyBookApprove::getCreateTime));
            // 获取文件并转PDF
            ByteArrayOutputStream bosWord = new ByteArrayOutputStream();
            ossFile.fileGet(identifyBookApproveList.get(0).getPath(), bosWord);
            ServletOutputStream servletOutputStream = response.getOutputStream();
            byte[] pdfByte = Word2PdfAsposeUtil.docx2pdf(new ByteArrayInputStream(bosWord.toByteArray()));
            ByteArrayOutputStream baos = new ByteArrayOutputStream(pdfByte.length);
            baos.write(pdfByte, 0, pdfByte.length);
            baos.writeTo(servletOutputStream);
            String fileName = identifyBookApproveList.get(0).getFileName();
            response.setContentType(new MimetypesFileTypeMap().getContentType(fileName.substring(0, fileName.length() - 5) + ".pdf"));
            baos.close();
        }catch (Exception e){
            System.out.println("获取鉴定文书审批表异常" + e.getMessage());
            e.printStackTrace();
        }
    }

}
