package digital.laboratory.platform.identifybook.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import digital.laboratory.platform.common.core.constant.OSSDirectoryConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.identifybook.dto.EntrustCaseDTO;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.entity.IdentifyBook;
import digital.laboratory.platform.identifybook.entity.IdentifyBookRelevance;
import digital.laboratory.platform.identifybook.event.PushDataToLabsCareEvent;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import digital.laboratory.platform.identifybook.service.IIdentifyBookRelevanceService;
import digital.laboratory.platform.identifybook.service.IIdentifyBookService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import org.springframework.core.env.Environment;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.List;
import java.util.Scanner;
import java.util.stream.Collectors;

/**
 * <p>
 * 鉴定文书-制作/修改
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023/5/22 17:41
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Api(tags = "鉴定文书-制作/修改-相关接口")
@RequestMapping("/book/identifyBookComm")
public class IdentifyBookCommController {


    private final OssFile ossFile;

    private final IEntrustmentService entrustmentService;

    private final IIdentifyBookService identifyBookService;

    private final IIdentifyBookRelevanceService identifyBookRelevanceService;

    private final Environment env;

    private final SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");

    private final ApplicationContext applicationContext;

    @ApiOperation(value = "根据委托ID制作鉴定文书", notes = "根据委托ID制作鉴定文书")
    @ApiImplicitParam(name = "entrustId", value = "委托ID", required = true)
    @GetMapping("/makeIdentifyBook/{entrustId}")
    public R makeIdentifyBook(@PathVariable(value = "entrustId") String entrustId, HttpServletResponse response) {
        EntrustCaseDTO entrustCaseDTO = null;
        try {
            entrustCaseDTO = identifyBookService.buildIdentifyBook(entrustId);
        } catch (Exception e) {
            e.printStackTrace();
            return R.failed("生成鉴定文书异常, 请联系系统管理员！！！");
        }
        if (entrustCaseDTO == null) {
            return R.failed("实验数据不满足生成文书!");
        } else {
            applicationContext.publishEvent(new PushDataToLabsCareEvent(this, entrustId, Collections.synchronizedSet(CollUtil.newHashSet( 6))));
            return R.ok(entrustCaseDTO, "生成鉴定文书成功！");
        }
    }

    @ApiOperation(value = "根据委托ID和修改后的鉴定文书更新鉴定文书", notes = "根据委托ID和修改后的鉴定文书更新鉴定文书")
    @ApiImplicitParam(name = "entrustId", value = "委托ID", required = true)
    @PostMapping("/updateIdentifyBook/{entrustId}")
    public R updateIdentifyBook(@PathVariable(value = "entrustId") String entrustId,
                                @RequestPart("file") MultipartFile file, HttpServletResponse response) {
        try {
            Entrustment entrustment = entrustmentService.getById(entrustId);
            List<String> stringIdsList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                            .select(Entrustment::getId)
                            .eq(Entrustment::getIdentificationId, entrustment.getIdentificationId()))
                    .stream().map(m -> m.getId())
                    .collect(Collectors.toList());
            IdentifyBookRelevance one = identifyBookRelevanceService.getOne(new LambdaQueryWrapper<IdentifyBookRelevance>()
                    .in(IdentifyBookRelevance::getRelevanceBusinessId, stringIdsList));
            // 获取最后一个文书版本数据
            IdentifyBook identifyBook = identifyBookService.getMaxVersionIdentifyBook(one.getId());

            String version = "v" + new BigDecimal(identifyBook.getVersion().substring(1)).add(new BigDecimal("0.1")); // 版本号
            String fileName = "鉴定文书-" + version;
            String fileType = ".docx"; // 文件类型
            String identifyBookPathFileName = OSSDirectoryConstants.IDENTIFY_BOOK_DIRECTORY + "/" + one.getRelevanceBusinessId() + "/book/" + fileName; // 鉴定文书存储路径及文件名
            // 上传文书
            ossFile.fileSave(identifyBookPathFileName + fileType, file.getInputStream());
            // 保存文书相关消息
            boolean b = this.saveIdentifyBookComm(one.getId(), identifyBookPathFileName + fileType, fileName + fileType, version, "修改文书");
            if (b) {
//                ossFile.fileGet(identifyBookPathFileName + fileType, response.getOutputStream());
//                response.setContentType(new MimetypesFileTypeMap().getContentType(fileName + fileType));
                String uuid = IdWorker.get32UUID();
                return R.ok(uuid + "#" + identifyBookPathFileName + fileType, "获取文件成功");
            }
        } catch (Exception e) {
            return R.failed("修改文书上传异常，请联系管理员！！！");
        }

        return R.failed("修改文书上传异常，请联系管理员！！！");
    }

    @ApiOperation(value = "根据委托ID和修改后的鉴定文书更新鉴定文书", notes = "根据委托ID和修改后的鉴定文书更新鉴定文书")
    @ApiImplicitParam(name = "entrustId", value = "委托ID", required = true)
    @PostMapping("/callBackIdentifyBook/{entrustId}")
    public void callBackIdentifyBook(@PathVariable(value = "entrustId") String entrustId, HttpServletRequest request, HttpServletResponse response) {
        log.error("entrustId:: " + entrustId);
        PrintWriter writer = null;
        JSONObject jsonObj = null;
        try {
            writer = response.getWriter();
            Scanner scanner = new Scanner(request.getInputStream()).useDelimiter("\\A");
            String body = scanner.hasNext() ? scanner.next() : "";
            jsonObj = JSONUtil.parseObj(body);
            System.out.println(jsonObj);
            System.out.println("===saveeditedfile:" + jsonObj.get("status"));
	            /*
	                0 - no document with the key identifier could be found,
	                1 - document is being edited,
	                2 - document is ready for saving,
	                3 - document saving error has occurred,
	                4 - document is closed with no changes,
	                6 - document is being edited, but the current document state is saved,
	                7 - error has occurred while force saving the document.
	             * */
            if ((long) jsonObj.get("status") == 2) {

                String downloadUri = (String) jsonObj.get("url");
                URL url = new URL(downloadUri);
                HttpURLConnection connection = (HttpURLConnection) url.openConnection();
                InputStream inputStream = connection.getInputStream();
                Entrustment entrustment = entrustmentService.getById(entrustId);
                List<String> stringIdsList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                                .select(Entrustment::getId)
                                .eq(Entrustment::getIdentificationId, entrustment.getIdentificationId()))
                        .stream().map(m -> m.getId())
                        .collect(Collectors.toList());
                IdentifyBookRelevance one = identifyBookRelevanceService.getOne(new LambdaQueryWrapper<IdentifyBookRelevance>()
                        .in(IdentifyBookRelevance::getRelevanceBusinessId, stringIdsList));
                // 获取最后一个文书版本数据
                IdentifyBook identifyBook = identifyBookService.getMaxVersionIdentifyBook(one.getId());

                String version = "v" + new BigDecimal(identifyBook.getVersion().substring(1)).add(new BigDecimal("0.1")); // 版本号
                String fileName = "鉴定文书-" + version;
                String fileType = ".docx"; // 文件类型
                String identifyBookPathFileName = OSSDirectoryConstants.IDENTIFY_BOOK_DIRECTORY + "/" + one.getRelevanceBusinessId() + "/book/" + fileName; // 鉴定文书存储路径及文件名
                // 上传文书
                ByteArrayOutputStream bosWord = new ByteArrayOutputStream();
                int readPos;
                final byte[] bytes_oss = new byte[1024];
                while ((readPos = inputStream.read(bytes_oss)) != -1) {
                    bosWord.write(bytes_oss, 0, readPos);
                }
                ByteArrayInputStream baisWord = new ByteArrayInputStream(bosWord.toByteArray());
                boolean b1 = ossFile.fileSave(identifyBookPathFileName + fileType, baisWord);
                if (b1) {
                    // 保存文书相关消息
                    this.saveIdentifyBookComm(one.getId(), identifyBookPathFileName + fileType, fileName + fileType, version, "修改文书");
                }

                inputStream.close();
                connection.disconnect();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        /**
         * status = 1，我们给onlyoffice的服务返回{"error":"0"}的信息，这样onlyoffice会认为回调接口是没问题的，这样就可以在线编辑文档了，否则的话会弹出窗口说明
         * 在线编辑还没有关闭，前端有人下载文档时，强制保存最新内容  当status 是6时说明有人在编辑时下载文档
         */
        System.out.println(jsonObj.get("status"));
        if ((long) jsonObj.get("status") == 6) {
            //处理当文档正在编辑为关闭时，下载文档
            if (((String) jsonObj.get("userdata")).equals("sample userdata")) {
                // FileUtil.callBackSaveDocument(jsonObj,filePath,request, response);
            }

            System.out.println("====保存失败:");
            writer.write("{\"error\":1}");
        } else {
            //执行删除编辑时下载保存的文件:
            // FileUtil.deleteTempFile(filePath,request.getParameter("fileName"));
            writer.write("{\"error\":0}");
        }

    }

    /**
     * 保存鉴定文书相关数据
     *
     * @param identifyBookRelevanceId 关联ID
     * @param path                    文件路径及文件名
     * @param fileName                文件名
     * @param version                 版本号
     * @return boolean
     */
    private boolean saveIdentifyBookComm(String identifyBookRelevanceId, String path, String fileName,
                                         String version, String remarks) {
        IdentifyBook identifyBook = new IdentifyBook();
        identifyBook.setIdentifyBookRelevanceId(identifyBookRelevanceId);
        identifyBook.setBucketName(env.getProperty("oss.bucket-name"));
        identifyBook.setPath(path);
        identifyBook.setFileName(fileName);
        identifyBook.setVersion(version);
        identifyBook.setRemarks(remarks);
        return identifyBookService.save(identifyBook);
    }
}
