package digital.laboratory.platform.identifybook.controller;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.cloud.commons.lang.StringUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.identifybook.entity.*;
import digital.laboratory.platform.identifybook.service.*;
import digital.laboratory.platform.sys.vo.CommonDataVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 鉴定数据-控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023/5/18 16:35
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Api(tags = "鉴定数据相关接口")
@RequestMapping("/book/identifyDataComm")
public class IdentifyDataCommController {

    /** TODO START 委托相关基础信息 **/
    private final ICaseEventService caseEventService;
    private final ICaseEvidenceService caseEvidenceService;
    private final IDelivererService delivererService;
    private final IEntrustmentService entrustmentService;
    private final IEntrustmentIdentificationMaterialService entrustmentIdentificationMaterialService;
    private final IHairJobService hairJobService;
    private final IHairJobIdentificationMaterialService hairJobIdentificationMaterialService;
    private final IIdentificationService identificationService;
    private final ISampleService sampleService;
    private final ITakerService takerService;
    /** TODO END 委托相关基础信息 **/

    private final IEntrustmentBundleService entrustmentBundleService;
    private final IExperimentService experimentService;
    private final IExaminationMethodsService examinationMethodsService;
    private final IResultEntrustmentQualitativeService resultEntrustmentQualitativeService;
    private final IResultEntrustmentRationService resultEntrustmentRationService;
    private final IBIdentificationProcessService identificationProcessService;

    private final IIdentifyBookRelevanceService identifyBookRelevanceService;



    @ApiOperation(value = "保存委托相关数据", notes = "保存委托相关数据")
    @PostMapping("/commonEntrustRelated")
    public R commonEntrustRelated(@RequestBody CommonDataVO commonDataVO){
        log.info("委托相关信息{}", commonDataVO);
        try {
            if(ObjectUtils.isNotEmpty(commonDataVO)){
                // 委托信息    检验过程  鉴定状态表
                if(CollectionUtils.isNotEmpty(commonDataVO.getEntrustmentList())){
                    log.info("委托信息：{}", commonDataVO.getEntrustmentList());
                    List<Entrustment> entrustmentList = BeanUtil.copyToList(commonDataVO.getEntrustmentList(), Entrustment.class);
                    boolean b = entrustmentService.saveOrUpdateBatch(entrustmentList);
                    if (b) {
                        Entrustment entrustment;
                        if(entrustmentList.size() > 1){
                            entrustment = entrustmentList
                                    .stream()
                                    .sorted(Comparator.comparing(Entrustment::getCreateTime))
                                    .collect(Collectors.toList()).get(0);
                        } else {
                            entrustment = entrustmentList.get(0);
                        }
                        List<String> stringIDSList = entrustmentList.stream().map(m -> m.getId()).collect(Collectors.toList());
                        IdentifyBookRelevance identifyBookRelevance = identifyBookRelevanceService.getOne(new LambdaQueryWrapper<IdentifyBookRelevance>()
                                .in(IdentifyBookRelevance::getRelevanceBusinessId, stringIDSList));
                        if(ObjectUtils.isEmpty(identifyBookRelevance)){
                            String[] arrNo = entrustment.getAcceptNo().split("-");
                            identifyBookRelevance = new IdentifyBookRelevance();
                            identifyBookRelevance.setRelevanceBusinessId(entrustment.getId());
                            identifyBookRelevance.setYear(arrNo[0]);
                            identifyBookRelevance.setSerialNumber(arrNo[1]);
                            identifyBookRelevance.setBookStatus(-1);
                            boolean save = identifyBookRelevanceService.save(identifyBookRelevance);
                            if(!save){
                                throw new RuntimeException("保存文书关联消息失败！！！");
                            }
                        }
                        // 检验过程
                        BIdentificationProcess identificationProcess = new BIdentificationProcess();
                        identificationProcess.setId(entrustment.getId());
                        identificationProcess.setInspectionProcess(
                                StringUtils.isNotEmpty(commonDataVO.getIdentificationProcess()) ? commonDataVO.getIdentificationProcess() : "");
                        boolean saveOrUpdate = identificationProcessService.saveOrUpdate(identificationProcess);
                        if(!saveOrUpdate){
                            throw new RuntimeException("保存文书关联检验过程失败！！！");
                        }
                        log.info("保存或更新委托成功！！！");
                    }

                }
                // 委托关联案件信息    --
                if(ObjectUtils.isNotEmpty(commonDataVO.getCaseEvent())){
                    log.info("委托关联案件信息：{}", commonDataVO.getCaseEvent());
                    CaseEvent caseEvent = new CaseEvent();
                    BeanUtil.copyProperties(commonDataVO.getCaseEvent(), caseEvent);
                    boolean b = caseEventService.saveOrUpdate(caseEvent);
                    if (b) {
                        log.info("保存或更新案件成功！！！");
                    }
                }
                // 委托案件关联物证    --
                if(CollectionUtils.isNotEmpty(commonDataVO.getCaseEvidence())){
                    log.info("委托案件关联物证：{}", commonDataVO.getCaseEvidence());
                    List<CaseEvidence> caseEvidenceList = BeanUtil.copyToList(commonDataVO.getCaseEvidence(), CaseEvidence.class);
                    boolean b = caseEvidenceService.saveOrUpdateBatch(caseEvidenceList);
                    if (b) {
                        log.info("保存或更新委托物证成功！！！");
                    }
                }
                // 检材信息
                if(CollectionUtils.isNotEmpty(commonDataVO.getEntrustmentIdentificationMaterial())){
                    log.info("检材信息：{}", commonDataVO.getEntrustmentIdentificationMaterial());
                    List<EntrustmentIdentificationMaterial> entrustmentIdentificationMaterialList =
                            BeanUtil.copyToList(commonDataVO.getEntrustmentIdentificationMaterial(), EntrustmentIdentificationMaterial.class);
                    boolean b = entrustmentIdentificationMaterialService.saveOrUpdateBatch(entrustmentIdentificationMaterialList);
                    if (b) {
                        log.info("保存或更新检材消息成功！！！");
                    }
                }
                // 鉴定表,一个鉴定可能会有多个委托
                if(ObjectUtils.isNotEmpty(commonDataVO.getIdentification())){
                    log.info("鉴定表,一个鉴定可能会有多个委托：{}", commonDataVO.getIdentification());
                    Identification identification = new Identification();
                    BeanUtil.copyProperties(commonDataVO.getIdentification(), identification);
                    boolean b = identificationService.saveOrUpdate(identification);
                    if (b) {
                        log.info("保存或更新鉴定表成功！！！");
                    }
                }
                // 检验用的样本
                if(CollectionUtils.isNotEmpty(commonDataVO.getSample())){
                    log.info("检验用的样本：{}", commonDataVO.getSample());
                    List<Map> copyToList = BeanUtil.copyToList(commonDataVO.getSample(), Map.class);
//                    List<Sample> sampleList = BeanUtil.copyToList(commonDataVO.getSample(), Sample.class);
                    List<Sample> sampleList = new ArrayList<>();
                    for (Map map: copyToList){
                        log.info("map.get(sampleInfo): {}",map.get("sampleInfo"));
//                        sampleList.add(BeanUtil.copyProperties(map.get("sampleInfo"), Sample.class));
                        //传递过来的数据中 sampleInfo可能为空
                        if(map.get("sampleInfo")!=null){
                            sampleList.add(JSON.parseObject(JSON.toJSONString(map.get("sampleInfo")), Sample.class));
                        }else{
                            log.info("map.get(sampleInfo): {},sampleId={} experimentId={} 这个为空",map.get("sampleInfo"),map.get("sampleId"),map.get("experimentId"));
                        }

                    }

                    log.info("检验用的样本sampleList：{}", sampleList);
                    boolean b = sampleService.saveOrUpdateBatch(sampleList);
                    log.info("检验用的样本sampleList-当这里么!!!");
                    if (b) {
                        log.info("保存或更新检验用的样本成功！！！");
                    }
                }
                // 委托与实验关系
                if(ObjectUtils.isNotEmpty(commonDataVO.getEntrustmentBundle())){
                    log.info("委托与实验关系：{}", commonDataVO.getEntrustmentBundle());
                    EntrustmentBundle entrustmentBundle = new EntrustmentBundle();
                    BeanUtil.copyProperties(commonDataVO.getEntrustmentBundle(), entrustmentBundle);
                    boolean b = entrustmentBundleService.saveOrUpdate(entrustmentBundle);
                    if (b) {
                        log.info("保存或更新委托与实验关系成功！！！");
                    }
                }
                // 实验列表
                if(CollectionUtils.isNotEmpty(commonDataVO.getExperimentList())){
                    log.info("实验列表：{}", commonDataVO.getExperimentList());
                    List<Experiment> experimentList = BeanUtil.copyToList(commonDataVO.getExperimentList(), Experiment.class);
                    boolean b = experimentService.saveOrUpdateBatch(experimentList);
                    if (b) {
                        log.info("保存或更新实验列表成功！！！");
                    }
                }
                // 检验方法列表
                if(CollectionUtils.isNotEmpty(commonDataVO.getMethodList())){
                    log.info("检验方法列表：{}", commonDataVO.getMethodList());
                    List<ExaminationMethods> examinationMethodsList = BeanUtil.copyToList(commonDataVO.getMethodList(), ExaminationMethods.class);
                    boolean b = examinationMethodsService.saveOrUpdateBatch(examinationMethodsList);
                    if (b) {
                        log.info("保存或更新检验方法列表成功！！！");
                    }
                }
                // 定性检验结果
                if(CollectionUtils.isNotEmpty(commonDataVO.getResultEntrustmentQualitative())){
                    log.info("定性检验结果：{}", commonDataVO.getResultEntrustmentQualitative());
                    List<ResultEntrustmentQualitative> resultEntrustmentQualitativeList =
                            BeanUtil.copyToList(commonDataVO.getResultEntrustmentQualitative(), ResultEntrustmentQualitative.class);
                    boolean b = resultEntrustmentQualitativeService.saveOrUpdateBatch(resultEntrustmentQualitativeList);
                    if (b) {
                        log.info("保存或更新定性检验结果！！！");
                    }
                }
                // 定量检验结果
                if(CollectionUtils.isNotEmpty(commonDataVO.getResultEntrustmentRation())){
                    log.info("定量检验结果：{}", commonDataVO.getResultEntrustmentRation());
                    List<ResultEntrustmentRation> resultEntrustmentRationList =
                            BeanUtil.copyToList(commonDataVO.getResultEntrustmentRation(), ResultEntrustmentRation.class);
                    boolean b = resultEntrustmentRationService.saveOrUpdateBatch(resultEntrustmentRationList);
                    if (b) {
                        log.info("保存或更新定量检验结果！！！");
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            return R.failed("保存或更新委托及鉴定结果异常，请联系管理员!!!");
        }

        return R.ok();
    }
}
