package digital.laboratory.platform.identifybook.controller;

import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.identifybook.emums.BusinessStatus;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.entity.IdentifyBookApprove;
import digital.laboratory.platform.identifybook.entity.ModifySupplementApply;
import digital.laboratory.platform.identifybook.req.ModifySupplementApplyREQ;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import digital.laboratory.platform.identifybook.service.IModifySupplementApplyService;
import digital.laboratory.platform.identifybook.utils.Word2PdfAsposeUtil;
import digital.laboratory.platform.sys.feign.RemoteUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.activation.MimetypesFileTypeMap;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.security.Principal;
import java.util.Arrays;
import java.util.Date;

/**
 * <p>
 * 鉴定文书修改补充申请表 前端控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-05-22
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "鉴定文书修改补充申请相关接口")
@RequestMapping("/book/modifySupplementApply")
public class ModifySupplementApplyController {

    private final OssFile ossFile;

    private final IModifySupplementApplyService modifySupplementApplyService;

    private final RemoteUserService remoteUserService;

    private final IEntrustmentService entrustmentService;

    @ApiOperation(value = "保存/更新鉴定文书修改补充申请信息", notes = "保存/更新鉴定文书修改补充申请信息")
    @PostMapping
    public R<ModifySupplementApply> addOrUpdate(@RequestBody ModifySupplementApply modifySupplementApply, HttpServletRequest request){
        // 获取用户消息
        Principal principal = request.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        // 查询当前是否已经申请并且还没有审批完成的记录
        ModifySupplementApply existInfo = modifySupplementApplyService
                .getOne(Wrappers.<ModifySupplementApply>lambdaQuery()
                        .eq(ModifySupplementApply::getEntrustmentId, modifySupplementApply.getEntrustmentId())
                        .eq(ModifySupplementApply::getApplyBy, dlpUser.getId())
                        .notIn(ModifySupplementApply::getStatus,
                                Arrays.asList(BusinessStatus.JD_WS_XG_BC_SP_TG.getCode(), BusinessStatus.JD_WS_XG_BC_SP_WTG.getCode())));
        if (existInfo != null) {
            throw new RuntimeException("当前您有未审批完成的申请，请先处理当前申请！");
        }
        Entrustment entrustment = entrustmentService.getById(modifySupplementApply.getEntrustmentId());
        String[] split = entrustment.getAcceptNo().split("-");
        modifySupplementApply.setApplyBy(dlpUser.getId());
        modifySupplementApply.setYear(split[0]);
        modifySupplementApply.setSerialNumber(split[1]);
        modifySupplementApply.setStatus(0);
        modifySupplementApply.setApplyTime(new Date());
        modifySupplementApply.setUpdateTime(new Date());
        if(modifySupplementApplyService.saveOrUpdate(modifySupplementApply)){
            return R.ok(modifySupplementApply);
        }
        return R.failed("保存或更新失败");
    }

    @ApiOperation(value = "获取当前用户是否有未审批的申请修改记录信息", notes = "获取当前用户是否有未审批的申请修改记录信息")
    @GetMapping("/getNotApproveApplyInfo")
    public R<ModifySupplementApply> getNotApproveApplyInfoByUser(@RequestParam String entrustId, HttpServletRequest request){
        // 获取用户消息
        Principal principal = request.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        // 查询当前是否已经申请并且还没有审批完成的记录
        ModifySupplementApply existInfo = modifySupplementApplyService
                .getOne(Wrappers.<ModifySupplementApply>lambdaQuery()
                        .eq(ModifySupplementApply::getEntrustmentId, entrustId)
                        .eq(ModifySupplementApply::getApplyBy, dlpUser.getId())
                        .notIn(ModifySupplementApply::getStatus,
                                Arrays.asList(BusinessStatus.JD_WS_XG_BC_SP_TG.getCode(), BusinessStatus.JD_WS_XG_BC_SP_WTG.getCode())));
        // 如果查询为空则new一个实例返回
        return R.ok(existInfo == null ? new ModifySupplementApply() : existInfo);
    }

    @ApiOperation(value = "通过鉴定文书修改补充申请ID删除鉴定文书修改补充申请信息", notes = "通过鉴定文书修改补充申请ID删除鉴定文书修改补充申请信息")
    @DeleteMapping("/{id}")
    public R<String> del(@PathVariable("id") String id){
        if(modifySupplementApplyService.removeById(id)){
            return R.ok();
        }
        return R.failed("删除鉴定文书修改补充申请信息失败！！！");
    }

    @ApiOperation(value = "通过鉴定文书修改补充申请ID查询鉴定文书修改补充申请信息", notes = "通过鉴定文书修改补充申请ID查询职鉴定文书修改补充申请信息")
    @GetMapping("/view/{id}")
    public R<ModifySupplementApply> view(@PathVariable("id") String id){
        ModifySupplementApply modifySupplementApply = modifySupplementApplyService.getById(id);
        if(ObjectUtils.isNotEmpty(modifySupplementApply)){
            if(StringUtils.isNotEmpty(modifySupplementApply.getEntrustBy()) && StringUtils.isEmpty(modifySupplementApply.getEntrustUserName())){
                modifySupplementApply.setEntrustUserName(
                        remoteUserService.innerGetById(modifySupplementApply.getEntrustBy()).getData().getName());
            }

            return R.ok(modifySupplementApply);
        }
        return R.failed("查询鉴定文书修改补充申请信息失败");
    }

    @ApiOperation(value = "通过条件分页查询鉴定文书修改补充申请信息", notes = "通过条件分页查询鉴定文书修改补充申请信息")
    @PostMapping("/search")
    public R<IPage<ModifySupplementApply>> getCareerCredentialsPage(@RequestBody ModifySupplementApplyREQ req){
        IPage<ModifySupplementApply> modifySupplementApplyIPage = modifySupplementApplyService.page(
                req.getPage(), null);
        if(CollectionUtils.isNotEmpty(modifySupplementApplyIPage.getRecords())){
            for (ModifySupplementApply modifySupplementApply: modifySupplementApplyIPage.getRecords()) {
                if(StringUtils.isNotEmpty(modifySupplementApply.getEntrustBy())){
                    modifySupplementApply.setEntrustUserName(
                            remoteUserService.innerGetById(modifySupplementApply.getEntrustBy()).getData().getName());
                }
            }
        }
        return R.ok(modifySupplementApplyIPage);
    }

    @ApiOperation(value = "根据鉴定文书修改补充申请ID获取鉴定文书修改补充申请文件", notes = "根据鉴定文书修改补充申请ID获取鉴定文书修改补充申请文件")
    @GetMapping("/getIdentifyBookApprove/{id}")
    public R getModifySupplementApplyWord(@PathVariable(value = "id") String id, HttpServletResponse response){
        try {
            ModifySupplementApply modifySupplementApply = modifySupplementApplyService.getById(id);
            if (ObjectUtils.isNotEmpty(modifySupplementApply) && StringUtils.isNotEmpty(modifySupplementApply.getPath())){

                ByteArrayOutputStream bosWord = new ByteArrayOutputStream();
                ossFile.fileGet(modifySupplementApply.getPath(), bosWord);
//                ossFile.fileGet(modifySupplementApply.getPath(), response.getOutputStream());
                ServletOutputStream servletOutputStream = response.getOutputStream();
                byte[] pdfByte = Word2PdfAsposeUtil.docx2pdf(new ByteArrayInputStream(bosWord.toByteArray()));
                ByteArrayOutputStream baos = new ByteArrayOutputStream(pdfByte.length);
                baos.write(pdfByte, 0, pdfByte.length);
                baos.writeTo(servletOutputStream);
                response.setContentType(new MimetypesFileTypeMap().getContentType(
                        modifySupplementApply.getFileName().substring(0, modifySupplementApply.getFileName().length() - 5) + ".pdf"));
//                String uuid= IdWorker.get32UUID();
//                return R.ok( uuid+"#"+modifySupplementApply.getPath(), "获取文件成功");
            }
        }catch (Exception e){
            System.out.println("获取鉴定文书审批表异常" + e.getMessage());
            e.printStackTrace();
        }
        return R.failed("获取鉴定文书修改补充申请文件失败");
    }

}
