package digital.laboratory.platform.identifybook.controller;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.identifybook.dto.EntrustmentDTO;
import digital.laboratory.platform.identifybook.dto.ReceiveIdentifyBookDTO;
import digital.laboratory.platform.identifybook.emums.BusinessStatus;
import digital.laboratory.platform.identifybook.entity.CaseEvent;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.entity.EntrustmentIdentificationMaterial;
import digital.laboratory.platform.identifybook.entity.ReceiveIdentifyBook;
import digital.laboratory.platform.identifybook.req.EntrustmentREQ;
import digital.laboratory.platform.identifybook.service.ICaseEventService;
import digital.laboratory.platform.identifybook.service.IEntrustmentIdentificationMaterialService;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import digital.laboratory.platform.identifybook.service.IReceiveIdentifyBookService;
import digital.laboratory.platform.identifybook.utils.BaseRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 领取鉴定文书信息表 前端控制器
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-08-30
 */
@RestController
@RequiredArgsConstructor
@Api(tags = "领取鉴定文书信息-相关接口")
@RequestMapping("/book/receiveIdentifyBook")
public class ReceiveIdentifyBookController {

    private final IEntrustmentService entrustmentService;

    private final ICaseEventService caseEventService;

    private final IEntrustmentIdentificationMaterialService entrustmentIdentificationMaterialService;

    private final IReceiveIdentifyBookService receiveIdentifyBookService;

    @ApiOperation(value = "根据条件分页查询委托数据列表", notes = "根据条件分页查询委托数据列表")
    @PostMapping("/findReceiveIdentifyBookPage")
    public R<IPage<ReceiveIdentifyBookDTO>> findEntrustPage(@RequestBody EntrustmentREQ req){
        if(req.getBookStatusList().size() == 1){
            if(req.getBookStatusList().get(0).equals("4")){
                IPage<EntrustmentDTO> entrustPage = entrustmentService.findEntrustPage(req.getPage(), req);
                IPage<ReceiveIdentifyBookDTO> bookDTOIPage = new Page<>();
                bookDTOIPage.setCurrent(entrustPage.getCurrent());
                bookDTOIPage.setPages(entrustPage.getPages());
                bookDTOIPage.setSize(entrustPage.getSize());
                bookDTOIPage.setTotal(entrustPage.getTotal());
                List<ReceiveIdentifyBookDTO> receiveIdentifyBookDTOList = new ArrayList<>();
                for (EntrustmentDTO entrustmentDTO: entrustPage.getRecords()){
                    ReceiveIdentifyBookDTO receiveIdentifyBookDTO = new ReceiveIdentifyBookDTO();
                    List<String> stringIDSList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                                    .select(Entrustment::getId)
                                    //.eq(Entrustment::getIdentificationId, entrustmentDTO.getIdentificationId())
                                    .eq(Entrustment::getId, entrustmentDTO.getId())
                            )
                            .stream().map(m -> m.getId())
                            .collect(Collectors.toList());
                    receiveIdentifyBookDTO.setEntrustId(entrustmentDTO.getId());
                    receiveIdentifyBookDTO.setSampleNum(entrustmentDTO.getMaterialNum());
                    BeanUtil.copyProperties(entrustmentDTO, receiveIdentifyBookDTO);
                 /*   List<ReceiveIdentifyBook> receiveIdentifyBookList = receiveIdentifyBookService.list(new LambdaQueryWrapper<ReceiveIdentifyBook>()
                            .in(ReceiveIdentifyBook::getRelevanceBusinessId, stringIDSList));
                    if(CollectionUtils.isNotEmpty(receiveIdentifyBookList)){
                        BeanUtil.copyProperties(receiveIdentifyBookList.get(0), receiveIdentifyBookDTO);
                        receiveIdentifyBookDTO.setReceiveId(receiveIdentifyBookList.get(0).getId());
                        receiveIdentifyBookDTO.setStatus(2);
                    } else {
                        receiveIdentifyBookDTO.setStatus(1);
                    }*/
                    receiveIdentifyBookDTO.setCaseName(entrustmentDTO.getCaseName());
                    receiveIdentifyBookDTO.setAcceptNo(entrustmentDTO.getAcceptNo());
                    receiveIdentifyBookDTO.setAcceptTime(entrustmentDTO.getAcceptTime());
                    receiveIdentifyBookDTO.setStatus(1);
                    if (StrUtil.isBlank(receiveIdentifyBookDTO.getReceiveWay())) {
                        receiveIdentifyBookDTO.setReceiveWay(entrustmentDTO.getReportReceiveMode());//优先使用原来的领取方式
                        receiveIdentifyBookDTO.setPostAddress(entrustmentDTO.getPostAddress());
                    }
                    receiveIdentifyBookDTOList.add(receiveIdentifyBookDTO);
                }
                bookDTOIPage.setRecords(receiveIdentifyBookDTOList);
                return R.ok(bookDTOIPage);
            } else {
                BaseRequest<ReceiveIdentifyBook> request = new BaseRequest<>();
                request.setCurrent(req.getCurrent());
                request.setSize(req.getSize());
                IPage<ReceiveIdentifyBookDTO> receiveIdentifyBookDTOIPage =
                        receiveIdentifyBookService.pageReceiveIdentifyBook(request.getPage(), req.getKey());
                if(CollectionUtils.isNotEmpty(receiveIdentifyBookDTOIPage.getRecords())){
                    for (ReceiveIdentifyBookDTO receiveIdentifyBookDTO: receiveIdentifyBookDTOIPage.getRecords()){
                        Entrustment entrustment = entrustmentService.getById(receiveIdentifyBookDTO.getEntrustId());
                        List<String> stringIDSList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                                        .select(Entrustment::getId)
                                        //.eq(Entrustment::getIdentificationId, entrustment.getIdentificationId())
                                        .eq(Entrustment::getId, entrustment.getId())
                                )
                                .stream().map(m -> m.getId())
                                .collect(Collectors.toList());
                        receiveIdentifyBookDTO.setCaseName(entrustment.getCaseName());
                        receiveIdentifyBookDTO.setSampleNum(entrustment.getMaterialNum());
                        receiveIdentifyBookDTO.setAcceptTime(entrustment.getAcceptTime());
                        receiveIdentifyBookDTO.setAcceptNo(entrustment.getAcceptNo());
                        receiveIdentifyBookDTO.setStatus(2);
                        if (StrUtil.isBlank(receiveIdentifyBookDTO.getReceiveWay())) {
                            receiveIdentifyBookDTO.setReceiveWay(entrustment.getReportReceiveMode());//优先使用原来的领取方式
                            receiveIdentifyBookDTO.setPostAddress(entrustment.getPostAddress());
                        }

                    }

                }
                return R.ok(receiveIdentifyBookDTOIPage);
            }
        }
        return R.ok(null,"未查询到数据");

    }
    @ApiOperation(value = "根据委托ID查看发放领取详情", notes = "根据委托ID查看发放领取详情")
    @ApiImplicitParam(name = "entrustId", value = "委托ID" , required = true)
    @GetMapping("/view/{entrustId}")
    public R<ReceiveIdentifyBookDTO> view(@PathVariable(value = "entrustId") String entrustId){
        Entrustment entrustment = entrustmentService.getById(entrustId);
        List<String> stringIDSList = entrustmentService.list(new LambdaQueryWrapper<Entrustment>()
                        .select(Entrustment::getId)
                        .eq(Entrustment::getIdentificationId,
                                entrustment.getIdentificationId()))
                .stream().map(m -> m.getId())
                .collect(Collectors.toList());
        List<ReceiveIdentifyBook> receiveIdentifyBookList = receiveIdentifyBookService.list(
                new LambdaQueryWrapper<ReceiveIdentifyBook>()
                        .in(ReceiveIdentifyBook::getRelevanceBusinessId, stringIDSList));
        if(CollectionUtils.isNotEmpty(receiveIdentifyBookList)){
            ReceiveIdentifyBookDTO receiveIdentifyBookDTO = new ReceiveIdentifyBookDTO();
            BeanUtil.copyProperties(receiveIdentifyBookList.get(0), receiveIdentifyBookDTO);
            CaseEvent caseEvent = caseEventService.getById(entrustment.getCaseId());
            receiveIdentifyBookDTO.setReceiveId(receiveIdentifyBookList.get(0).getId());
            receiveIdentifyBookDTO.setEntrustId(receiveIdentifyBookList.get(0).getRelevanceBusinessId());
            receiveIdentifyBookDTO.setDeliverer1Name(entrustment.getDeliverer1Name());
            receiveIdentifyBookDTO.setDeliverer2Name(entrustment.getDeliverer2Name());
            receiveIdentifyBookDTO.setDeliverTime(entrustment.getDeliverTime());
            receiveIdentifyBookDTO.setIdentificationDomain(entrustment.getIdentificationDomain());
            receiveIdentifyBookDTO.setCaseNo(caseEvent.getCaseNo());
            receiveIdentifyBookDTO.setThirdPartySysNo(caseEvent.getThirdPartySysNo());
            receiveIdentifyBookDTO.setCaseName(caseEvent.getCaseName());
            receiveIdentifyBookDTO.setSampleNum(entrustment.getMaterialNum());
            return R.ok(receiveIdentifyBookDTO);
        }

        return R.failed("为获取到相关数据!!!!");
    }

}
