package digital.laboratory.platform.identifybook.listener;


import digital.laboratory.platform.identifybook.event.PushDataToLabsCareEvent;
import digital.laboratory.platform.identifybook.service.PushDataToLabsCareService;
import digital.laboratory.platform.identifybook.threadpool.GlobalThreadPool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.concurrent.CompletableFuture;

/**
 * 监听有关推送数据到LabsCare 平台的事件， 采用异步
 */
@Slf4j
@Component
public class PushDataToLabsCareEventListener implements ApplicationListener<PushDataToLabsCareEvent> {

    @Resource
    private PushDataToLabsCareService pushDataToLabsCareService;

    /**
     * 处理委托数据推送事件
     *
     * @param event 委托数据推送事件对象
     */
    @Override
    public void onApplicationEvent(PushDataToLabsCareEvent event) {
        CompletableFuture.runAsync(() -> {
            event.getPushType().forEach(pushType -> {
                // 处理不同的推送类型
                switch (pushType) {
                    case 5: // 推送鉴定报告审批表数据
                        try {
                            pushDataToLabsCareService.pushAppraisalReportApprovalFormData(event.getEntrustId());
                        } catch (Exception e) {
                            log.error("委托id为 {} 的鉴定报告审批表数据推送失败！", event.getEntrustId(), e);
                        }
                        break;
                    case 6: // 推送定性报告数据
                        try {
                            pushDataToLabsCareService.pushQualitativeReportData(event.getEntrustId());
                        } catch (Exception e) {
                            log.error("委托id为 {} 的定性报告数据推送失败！", event.getEntrustId(), e);
                        }
                        break;
                    case 7: // 推送定性报告数据
                        try {
                            pushDataToLabsCareService.pushArchiveReportInfoData(event.getEntrustId());
                        } catch (Exception e) {
                            log.error("委托id为 {} 的归档报告信息数据推送失败！", event.getEntrustId(), e);
                        }
                        break;
                    default:
                        log.warn("未知的推送类型: {}", event.getPushType());
                        break;
                }
            });
        }, GlobalThreadPool.getInstance()).exceptionally(e -> {
            log.error("推送数据到 LabsCare 失败", e);
            return null;
        });
    }

}
