package digital.laboratory.platform.identifybook.service.impl;

import com.alibaba.cloud.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.identifybook.entity.TaskNotification;
import digital.laboratory.platform.identifybook.mapper.TaskNotificationMapper;
import digital.laboratory.platform.identifybook.req.TaskNotificationREQ;
import digital.laboratory.platform.identifybook.service.ITaskNotificationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.identifybook.vo.TaskNotificationVO;
import org.springframework.stereotype.Service;

import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 任务信息表 服务实现类
 * </p>
 *
 * @author 旌霖科技-www.jinglin.com
 * @since 2023-08-31
 */
@Service
public class TaskNotificationServiceImpl extends ServiceImpl<TaskNotificationMapper, TaskNotification> implements ITaskNotificationService {

    /**
     * 统计待处理任务数量
     * @param dlpUser 当前用户消息
     * @return
     */
    @Override
    public long countRWSum(DLPUser dlpUser) {
        return this.count(new LambdaQueryWrapper<TaskNotification>()
                .in(TaskNotification::getStatus, Stream.of(0).collect(Collectors.toList()))
                .in(TaskNotification::getPermissionGroup, dlpUser.getPermissions()));
    }

    /**
     * 分页查询
     *
     * @param req
     * @param dlpUser
     * @return
     */
    @Override
    public IPage<TaskNotificationVO> queryTaskNotificationPage(TaskNotificationREQ req, DLPUser dlpUser) {
        return baseMapper.queryTaskNotificationPage(req.getPage(),
                new QueryWrapper<TaskNotification>()
                        .eq(req.getType() != null && req.getType() > 0, "tn.type", req.getType())
                        .in(CollectionUtils.isNotEmpty(req.getStatusList()), "tn.status", req.getStatusList())
                        .in("tn.permission_group", dlpUser.getPermissions())
                        .and(StringUtils.isNotEmpty(req.getKey()), wrapper -> {
                            wrapper.like("tn.title", req.getKey())
                                    .or()
                                    .like("tn.content", req.getKey());
                        })
                        .orderByDesc("tn.create_time"));
    }
}
