package digital.laboratory.platform.identifybook.utils;

import com.aspose.words.Document;
import com.aspose.words.License;
import com.aspose.words.SaveFormat;
import lombok.extern.slf4j.Slf4j;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

@Slf4j
public class Word2PdfAsposeUtil {
 
	
	public static boolean getLicense() {
        boolean result = false;  
        try {  
            //InputStream is = Test.class.getClassLoader().getResourceAsStream("\\license.xml"); // license.xml应放在..\WebRoot\WEB-INF\classes路径下
            InputStream is=Thread.currentThread().getContextClassLoader().getResourceAsStream("license.xml");
            License aposeLic = new License();
            aposeLic.setLicense(is);  
            result = true;  
        } catch (Exception e) {
            e.printStackTrace();  
        }  
        return result;  
    }

    public static byte[] docx2pdf(String urlString) throws MalformedURLException {
        InputStream inputStream = null;
        try {
            URL url = new URL(urlString);
            URLConnection urlCon = url.openConnection();
            inputStream = urlCon.getInputStream();
        } catch (Exception e){
            return null;
        }

        return docx2pdf(inputStream);
    }
  
    public static byte[] docx2pdf(InputStream is) {
        if (!getLicense()) { // 验证License 若不验证则转化出的pdf文档会有水印产生  
            return null;
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        byte[] bytes = null;
        try {
            long old = System.currentTimeMillis();
            Document doc = new Document(is); // Address是将要被转化的word文档
            doc.save(os, SaveFormat.PDF);// 全面支持DOC, DOCX, OOXML, RTF HTML, OpenDocument, PDF,
                                         // EPUB, XPS, SWF 相互转换
            bytes = os.toByteArray();
            long now = System.currentTimeMillis();
            System.out.println("pdf转换成功，共耗时：" + ((now - old) / 1000.0) + "秒"); // 转化用时
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }finally {
            if (os != null) {
                try {
                    os.flush();
                    os.close();
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return bytes;
    }
    public static ByteArrayOutputStream getPdfByteArrayOutputStream(InputStream is) {
        if (!getLicense()) { // 验证License 若不验证则转化出的pdf文档会有水印产生
            return null;
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        byte[] bytes = null;
        try {
            long old = System.currentTimeMillis();
            Document doc = new Document(is); // Address是将要被转化的word文档
            doc.save(os, SaveFormat.PDF);// 全面支持DOC, DOCX, OOXML, RTF HTML, OpenDocument, PDF,
            // EPUB, XPS, SWF 相互转换
            bytes = os.toByteArray();
            long now = System.currentTimeMillis();
            System.out.println("pdf转换成功，共耗时：" + ((now - old) / 1000.0) + "秒"); // 转化用时
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }finally {
            if (os != null) {
                try {
                    os.flush();
                    os.close();
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return os;
    }
}
